import { Plugin, ButtonView } from 'ckeditor5';

import ckeditor5Icon from '../theme/icons/ckeditor.svg';

import { asBlob } from 'html-docx-js-typescript';
import { saveAs } from 'file-saver';

import Temml from 'temml';

export default class Export extends Plugin {
	public static get pluginName() {
		return 'Export' as const;
	}

	public init(): void {
		const editor = this.editor;
		const t = editor.t;
		const model = editor.model;

		// Add the "exportButton" to feature components.
		editor.ui.componentFactory.add( 'exportButton', locale => {
			const view = new ButtonView( locale );

			view.set( {
				label: t( 'Export' ),
				icon: ckeditor5Icon,
				tooltip: true
			} );

			// // Insert a text into the editor after clicking the button.
			// this.listenTo( view, 'execute', () => {
			// 	model.change( writer => {
			// 		const textNode = writer.createText( 'Hello CKEditor 5!' );

			// 		model.insertContent( textNode );
			// 	} );

			// 	editor.editing.view.focus();
			// } );

			// Insert a text into the editor after clicking the button.
			// this.listenTo( view, 'execute', async () => {
			// 	asBlob(this.editor.getData()).then((data: any) => {
			// 		saveAs(data, 'file.docx') // save as docx file
			// 	}) // asBlob() return Promise<Blob|Buffer>

			// 	editor.editing.view.focus();
			// } );

			// POST fetch docx and download
			this.listenTo( view, 'execute', async () => {
				const parser = new DOMParser();
				const doc = parser.parseFromString(this.editor.getData(), "text/html");

				for (const mathliveElement of doc.querySelectorAll('script[type="math/tex"]')) {
					const temmlWrapperElement = document.createElement('span');
					const temmlMathMLString = Temml.render(mathliveElement.textContent ?? "", temmlWrapperElement);
					mathliveElement.replaceWith(temmlWrapperElement);
				}

				console.log(doc.body.innerHTML);

				fetch('http://localhost:3000/docx', {
					method: 'POST',
					body: JSON.stringify({ htmlData: this.editor.getData() }),
					headers: {
						'Content-Type': 'application/json'
					}
				})
				.then(response => response.blob())
				.then(blob => {
					saveAs(blob, 'stou.docx') // save as docx file
				})
				.catch(error => {
					console.error('Error:', error);
				});
				editor.editing.view.focus();
			} );

			return view;
		} );
	}
}
